############################################################
#   WebCompile.py  Version 1.0
#   Author: Laura Dietz (http://www.myselfine.de)
#
#   WebCompile is freeware
#   Commercial use not allowed
#   Modifications are allowed as long as this remark is included
############################################################


import string, glob
import os, sys, fnmatch
from stat import * 


############################################################
#       PLEASE CHANGE! (but do *not* insert leading blanks)
# baseurl = "http://www.mydomain.de/mydir/"
baseurl = "http://www.mydomain.de/mydir/"
#
############################################################

baseslashes = string.count(baseurl,'/')

def get_outfile(filename):
  '''replace in with out in path'''
  return string.joinfields(string.splitfields(filename,"/in/"), "/out/")

def convert_slash(filename):
  '''replace forward slashes with backslashes'''
  return string.joinfields(string.splitfields(filename,"/"),"\\")

def reconvert_slash(filename):
  '''replace forward slashes with backslashes'''
  return string.joinfields(string.splitfields(filename,"\\"),"/")

def get_title_from_titleline(titleline):
  tb = string.find(titleline,'<!-- ')
  te = string.find(titleline,' -->', tb)
  if((tb!=-1) & (te!=-1)):
    return titleline[tb+5:te]
  else:
    return 0


def process(dir, func_html, func_other, func_dir ):
  '''recursively descend the directory rooted at dir, calling func for each regular file'''
  dir_into(dir)
  for f in os.listdir(dir):
    mode = os.stat('%s/%s' % (dir, f))[ST_MODE]
    if S_ISDIR(mode):
      # recurse into directory
      print "DIR   "+f
      func_dir('%s/%s' % (dir, f))
      process('%s/%s' % (dir, f), func_html, func_other, func_dir)
      
    elif S_ISREG(mode):
      if fnmatch.fnmatch(f,"*.htm*"):
        print 'HTML  %s/%s' % (dir, f)
        func_html('%s/%s' % (dir, f))
      else:
        print 'OTHER %s/%s' % (dir, f)
        func_other('%s/%s' % (dir, f))
      
    else:
      print 'SKIP  %s/%s' % (dir, f)
  dir_outof()    

def process_dir(dirname):
  '''actions to be done with each dirname'''
  os.system("mkdir "+get_outfile(convert_slash(dirname)))

def dir_into(dirname):
  '''actions to be done while entering directory'''
  dirname=convert_slash(dirname)
  
  # Title
  headfile = open (dirname+"\\head.tmp",'ra')
  titleline = headfile.readline()
  title = get_title_from_titleline(titleline)
  titlelist.append(title)
  headfile.close()
  
  # Head
  headfile = open (dirname+"\\head.tmp",'ra')
  headlist.append(headfile.readlines())
  headfile.close()
  # Foot
  footfile = open (dirname+"\\foot.tmp",'ra')
  footlist.insert(0,footfile.readlines())
  footfile.close()
  

  
def dir_outof():
  '''actions to be done after leaving dir'''
  # delete last listentrys
  del headlist[len(headlist)-1]
  del titlelist[len(titlelist)-1]
  del footlist[0]  
  

def process_other_file(filename):
  '''actions to be done with each unspecified filetype'''
  if fnmatch.fnmatch(filename,"*.tmp"):
    pass
  else:
    os.system(convert_slash("copy "+filename+" "+get_outfile(filename)))

def change_html(line, outfilename, titles = "", thistitle = ""):
  # replace mutated vowels
  line = string.joinfields( string.splitfields(line,''), '&auml;')
  line = string.joinfields( string.splitfields(line,''), '&ouml;')
  line = string.joinfields( string.splitfields(line,''), '&uuml;')
  line = string.joinfields( string.splitfields(line,''), '&Auml;')
  line = string.joinfields( string.splitfields(line,''), '&Ouml;')
  line = string.joinfields( string.splitfields(line,''), '&Uuml;')
  
  # modify urls
  line = string.joinfields( string.splitfields(line,'href="/'), 'href="'+baseurl)
  line = string.joinfields( string.splitfields(line,'href = "/'), 'href = "'+baseurl)
  line = string.joinfields( string.splitfields(line,'src="/'), 'src="'+baseurl)
  line = string.joinfields( string.splitfields(line,'src = "/'), 'src = "'+baseurl)
  
  
  # insert special variables
  line = string.joinfields( string.splitfields(line,'%%titles%%'), titles)
  line = string.joinfields( string.splitfields(line,'%%thistitle%%'), thistitle)
  
 
  
  doc_url=baseurl+outfilename[6:]
 
  # delete url to _this_ file (works only for <a ..> </a> in the same line)
  # <a href="xxx"> blubb </a>
  # ^bb          ^be     ^eb^ee 
    
  doc_part_urls = string.splitfields(doc_url,"/")

  # http://www -> [http:],[],[www]
  i=baseslashes+1
  while i<=len(doc_part_urls):
    doc_part_url = string.joinfields(doc_part_urls[:i],"/")
    if(i<len(doc_part_urls)): doc_part_url = doc_part_url+"/"

    mb = string.find(line, doc_part_url+'"')
    while (mb>=0):
      
      bb = string.rfind(line[0:mb], '<a')
      be = string.find(line, '>', mb)
      eb = string.find(line, '</a>', be)
      ee = eb+3
      
      if((bb!=0)&(be!=0)&(be!=0)&(be<len(line))):
        # join non-link parts
        tmpline = line[0:bb]+line[be+1:eb]+line[ee+1:]
        line = tmpline
      else:
        print "ERROR: want to elimintate document URL, but <a>-Tag contains more than one line.... skipping."
        break
    
      mb = string.find(line, doc_part_url)
    i=i+1  
    
  return line  
  

def process_html(filename):
  '''actions to be done with each html file'''
  outfilename = get_outfile(filename)
  
  # get Title
  infile = open (filename,'ra')
  titleline = infile.readline();
  thistitle = get_title_from_titleline(titleline)
  if(thistitle == 0): thistitle = "" # no title defined
  infile.close()

  titles = ""
  for title in titlelist:
    if(title!=0):
      titles = titles + title + " - "

  titles = titles + thistitle
  

  infile = open (filename,'ra')
  outfile = open (outfilename, 'wa')

  outfile.write("<!-- This page was generated by WebCompile. WebCompile can be obtained from http://www.myselfine.de -->\n");

  # insert heads
  for head in headlist:
    for line in head:
      line = change_html(line = line, outfilename = outfilename, titles = titles, thistitle = thistitle)  
      outfile.write(line)

  while 1:
    # for each line in infile
    line = infile.readline();
    
    if not line:
      break
    else:
      line = change_html(line = line, outfilename = outfilename, titles = titles, thistitle = thistitle)  
      
      # write to file
      outfile.write(line)

  # insert feet
  for foot in footlist:
    for line in foot:
      line = change_html(line = line, outfilename = outfilename, titles = titles, thistitle = thistitle)  
      outfile.write(line)

  infile.close()
  outfile.close()




if __name__ == '__main__': 
  headlist = []
  footlist = []
  titlelist = []
  process(dir="./in", func_html=process_html, func_other=process_other_file, func_dir=process_dir)  
  
  
    
      